
#ifndef VTKPVVTKEXTENSIONSIOCORE_EXPORT_H
#define VTKPVVTKEXTENSIONSIOCORE_EXPORT_H

#ifdef VTKPVVTKEXTENSIONSIOCORE_STATIC_DEFINE
#  define VTKPVVTKEXTENSIONSIOCORE_EXPORT
#  define VTKPVVTKEXTENSIONSIOCORE_NO_EXPORT
#else
#  ifndef VTKPVVTKEXTENSIONSIOCORE_EXPORT
#    ifdef VTKExtensionsIOCore_EXPORTS
        /* We are building this library */
#      define VTKPVVTKEXTENSIONSIOCORE_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKPVVTKEXTENSIONSIOCORE_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKPVVTKEXTENSIONSIOCORE_NO_EXPORT
#    define VTKPVVTKEXTENSIONSIOCORE_NO_EXPORT 
#  endif
#endif

#ifndef VTKPVVTKEXTENSIONSIOCORE_DEPRECATED
#  define VTKPVVTKEXTENSIONSIOCORE_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKPVVTKEXTENSIONSIOCORE_DEPRECATED_EXPORT
#  define VTKPVVTKEXTENSIONSIOCORE_DEPRECATED_EXPORT VTKPVVTKEXTENSIONSIOCORE_EXPORT VTKPVVTKEXTENSIONSIOCORE_DEPRECATED
#endif

#ifndef VTKPVVTKEXTENSIONSIOCORE_DEPRECATED_NO_EXPORT
#  define VTKPVVTKEXTENSIONSIOCORE_DEPRECATED_NO_EXPORT VTKPVVTKEXTENSIONSIOCORE_NO_EXPORT VTKPVVTKEXTENSIONSIOCORE_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKPVVTKEXTENSIONSIOCORE_NO_DEPRECATED
#    define VTKPVVTKEXTENSIONSIOCORE_NO_DEPRECATED
#  endif
#endif

#endif /* VTKPVVTKEXTENSIONSIOCORE_EXPORT_H */
