
#ifndef VTKPVVTKEXTENSIONSCONDUIT_EXPORT_H
#define VTKPVVTKEXTENSIONSCONDUIT_EXPORT_H

#ifdef VTKPVVTKEXTENSIONSCONDUIT_STATIC_DEFINE
#  define VTKPVVTKEXTENSIONSCONDUIT_EXPORT
#  define VTKPVVTKEXTENSIONSCONDUIT_NO_EXPORT
#else
#  ifndef VTKPVVTKEXTENSIONSCONDUIT_EXPORT
#    ifdef VTKExtensionsConduit_EXPORTS
        /* We are building this library */
#      define VTKPVVTKEXTENSIONSCONDUIT_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKPVVTKEXTENSIONSCONDUIT_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKPVVTKEXTENSIONSCONDUIT_NO_EXPORT
#    define VTKPVVTKEXTENSIONSCONDUIT_NO_EXPORT 
#  endif
#endif

#ifndef VTKPVVTKEXTENSIONSCONDUIT_DEPRECATED
#  define VTKPVVTKEXTENSIONSCONDUIT_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKPVVTKEXTENSIONSCONDUIT_DEPRECATED_EXPORT
#  define VTKPVVTKEXTENSIONSCONDUIT_DEPRECATED_EXPORT VTKPVVTKEXTENSIONSCONDUIT_EXPORT VTKPVVTKEXTENSIONSCONDUIT_DEPRECATED
#endif

#ifndef VTKPVVTKEXTENSIONSCONDUIT_DEPRECATED_NO_EXPORT
#  define VTKPVVTKEXTENSIONSCONDUIT_DEPRECATED_NO_EXPORT VTKPVVTKEXTENSIONSCONDUIT_NO_EXPORT VTKPVVTKEXTENSIONSCONDUIT_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKPVVTKEXTENSIONSCONDUIT_NO_DEPRECATED
#    define VTKPVVTKEXTENSIONSCONDUIT_NO_DEPRECATED
#  endif
#endif

#endif /* VTKPVVTKEXTENSIONSCONDUIT_EXPORT_H */
