/*
 * Distributed under OSI-approved BSD 3-Clause License. See
 * accompanying License.txt
 */
#ifndef catalyst_version_h
#define catalyst_version_h

#define CATALYST_VERSION "2.0"
#define CATALYST_VERSION_MAJOR 2
#define CATALYST_VERSION_MINOR 0

#define CATALYST_ABI_VERSION "2"
#define CATALYST_ABI_VERSION_MAJOR 
#define CATALYST_ABI_VERSION_MINOR 

#define CATALYST_VERSION_CHECK(major, minor) \
  (10000000000ULL * major + 100000000ULL * minor)
#define CATALYST_VERSION_NUMBER                                                                         \
  CATALYST_VERSION_CHECK(CATALYST_MAJOR_VERSION, CATALYST_MINOR_VERSION)

#define CATALYST_ABI_VERSION_NUMBER \
  CATALYST_VERSION_CHECK(CATALYST_ABI_VERSION_MAJOR, CATALYST_ABI_VERSION_MINOR)
#endif
