#ifndef vtk_glew_mangle_h
#define vtk_glew_mangle_h

#define glewGetErrorString vtkglew_glewGetErrorString
#define glewGetExtension vtkglew_glewGetExtension
#define glewGetString vtkglew_glewGetString
#define glewInit vtkglew_glewInit
#define glewIsSupported vtkglew_glewIsSupported
#define glxewContextInit vtkglew_glxewContextInit
#define glxewGetExtension vtkglew_glxewGetExtension
#define glxewInit vtkglew_glxewInit
#define glxewIsSupported vtkglew_glxewIsSupported

#endif
