#ifndef vtk_tiff_mangle_h
#define vtk_tiff_mangle_h

#define LogL10fromY vtktiff_LogL10fromY
#define LogL10toY vtktiff_LogL10toY
#define LogL16fromY vtktiff_LogL16fromY
#define LogL16toY vtktiff_LogL16toY
#define LogLuv24fromXYZ vtktiff_LogLuv24fromXYZ
#define LogLuv24toXYZ vtktiff_LogLuv24toXYZ
#define LogLuv32fromXYZ vtktiff_LogLuv32fromXYZ
#define LogLuv32toXYZ vtktiff_LogLuv32toXYZ
#define TIFFAccessTagMethods vtktiff_TIFFAccessTagMethods
#define TIFFCIELabToRGBInit vtktiff_TIFFCIELabToRGBInit
#define TIFFCIELabToXYZ vtktiff_TIFFCIELabToXYZ
#define TIFFCheckTile vtktiff_TIFFCheckTile
#define TIFFCheckpointDirectory vtktiff_TIFFCheckpointDirectory
#define TIFFCleanup vtktiff_TIFFCleanup
#define TIFFClientOpen vtktiff_TIFFClientOpen
#define TIFFClientdata vtktiff_TIFFClientdata
#define TIFFClose vtktiff_TIFFClose
#define TIFFComputeStrip vtktiff_TIFFComputeStrip
#define TIFFComputeTile vtktiff_TIFFComputeTile
#define TIFFCreateCustomDirectory vtktiff_TIFFCreateCustomDirectory
#define TIFFCreateDirectory vtktiff_TIFFCreateDirectory
#define TIFFCreateEXIFDirectory vtktiff_TIFFCreateEXIFDirectory
#define TIFFCurrentDirOffset vtktiff_TIFFCurrentDirOffset
#define TIFFCurrentDirectory vtktiff_TIFFCurrentDirectory
#define TIFFCurrentRow vtktiff_TIFFCurrentRow
#define TIFFCurrentStrip vtktiff_TIFFCurrentStrip
#define TIFFCurrentTile vtktiff_TIFFCurrentTile
#define TIFFDataWidth vtktiff_TIFFDataWidth
#define TIFFDefaultDirectory vtktiff_TIFFDefaultDirectory
#define TIFFDefaultStripSize vtktiff_TIFFDefaultStripSize
#define TIFFDefaultTileSize vtktiff_TIFFDefaultTileSize
#define TIFFDeferStrileArrayWriting vtktiff_TIFFDeferStrileArrayWriting
#define TIFFError vtktiff_TIFFError
#define TIFFErrorExt vtktiff_TIFFErrorExt
#define TIFFFaxBlackCodes vtktiff_TIFFFaxBlackCodes
#define TIFFFaxBlackTable vtktiff_TIFFFaxBlackTable
#define TIFFFaxMainTable vtktiff_TIFFFaxMainTable
#define TIFFFaxWhiteCodes vtktiff_TIFFFaxWhiteCodes
#define TIFFFaxWhiteTable vtktiff_TIFFFaxWhiteTable
#define TIFFFdOpen vtktiff_TIFFFdOpen
#define TIFFFieldDataType vtktiff_TIFFFieldDataType
#define TIFFFieldName vtktiff_TIFFFieldName
#define TIFFFieldPassCount vtktiff_TIFFFieldPassCount
#define TIFFFieldReadCount vtktiff_TIFFFieldReadCount
#define TIFFFieldTag vtktiff_TIFFFieldTag
#define TIFFFieldWithName vtktiff_TIFFFieldWithName
#define TIFFFieldWithTag vtktiff_TIFFFieldWithTag
#define TIFFFieldWriteCount vtktiff_TIFFFieldWriteCount
#define TIFFFileName vtktiff_TIFFFileName
#define TIFFFileno vtktiff_TIFFFileno
#define TIFFFillStrip vtktiff_TIFFFillStrip
#define TIFFFillTile vtktiff_TIFFFillTile
#define TIFFFindCODEC vtktiff_TIFFFindCODEC
#define TIFFFindField vtktiff_TIFFFindField
#define TIFFFlush vtktiff_TIFFFlush
#define TIFFFlushData vtktiff_TIFFFlushData
#define TIFFFlushData1 vtktiff_TIFFFlushData1
#define TIFFForceStrileArrayWriting vtktiff_TIFFForceStrileArrayWriting
#define TIFFFreeDirectory vtktiff_TIFFFreeDirectory
#define TIFFGetBitRevTable vtktiff_TIFFGetBitRevTable
#define TIFFGetClientInfo vtktiff_TIFFGetClientInfo
#define TIFFGetCloseProc vtktiff_TIFFGetCloseProc
#define TIFFGetConfiguredCODECs vtktiff_TIFFGetConfiguredCODECs
#define TIFFGetField vtktiff_TIFFGetField
#define TIFFGetFieldDefaulted vtktiff_TIFFGetFieldDefaulted
#define TIFFGetMapFileProc vtktiff_TIFFGetMapFileProc
#define TIFFGetMode vtktiff_TIFFGetMode
#define TIFFGetReadProc vtktiff_TIFFGetReadProc
#define TIFFGetSeekProc vtktiff_TIFFGetSeekProc
#define TIFFGetSizeProc vtktiff_TIFFGetSizeProc
#define TIFFGetStrileByteCount vtktiff_TIFFGetStrileByteCount
#define TIFFGetStrileByteCountWithErr vtktiff_TIFFGetStrileByteCountWithErr
#define TIFFGetStrileOffset vtktiff_TIFFGetStrileOffset
#define TIFFGetStrileOffsetWithErr vtktiff_TIFFGetStrileOffsetWithErr
#define TIFFGetTagListCount vtktiff_TIFFGetTagListCount
#define TIFFGetTagListEntry vtktiff_TIFFGetTagListEntry
#define TIFFGetUnmapFileProc vtktiff_TIFFGetUnmapFileProc
#define TIFFGetVersion vtktiff_TIFFGetVersion
#define TIFFGetWriteProc vtktiff_TIFFGetWriteProc
#define TIFFInitCCITTFax3 vtktiff_TIFFInitCCITTFax3
#define TIFFInitCCITTFax4 vtktiff_TIFFInitCCITTFax4
#define TIFFInitCCITTRLE vtktiff_TIFFInitCCITTRLE
#define TIFFInitCCITTRLEW vtktiff_TIFFInitCCITTRLEW
#define TIFFInitDumpMode vtktiff_TIFFInitDumpMode
#define TIFFInitJPEG vtktiff_TIFFInitJPEG
#define TIFFInitLZW vtktiff_TIFFInitLZW
#define TIFFInitNeXT vtktiff_TIFFInitNeXT
#define TIFFInitPackBits vtktiff_TIFFInitPackBits
#define TIFFInitPixarLog vtktiff_TIFFInitPixarLog
#define TIFFInitSGILog vtktiff_TIFFInitSGILog
#define TIFFInitThunderScan vtktiff_TIFFInitThunderScan
#define TIFFInitZIP vtktiff_TIFFInitZIP
#define TIFFIsBigEndian vtktiff_TIFFIsBigEndian
#define TIFFIsByteSwapped vtktiff_TIFFIsByteSwapped
#define TIFFIsCODECConfigured vtktiff_TIFFIsCODECConfigured
#define TIFFIsMSB2LSB vtktiff_TIFFIsMSB2LSB
#define TIFFIsTiled vtktiff_TIFFIsTiled
#define TIFFIsUpSampled vtktiff_TIFFIsUpSampled
#define TIFFJPEGIsFullStripRequired vtktiff_TIFFJPEGIsFullStripRequired
#define TIFFLastDirectory vtktiff_TIFFLastDirectory
#define TIFFMergeFieldInfo vtktiff_TIFFMergeFieldInfo
#define TIFFNumberOfDirectories vtktiff_TIFFNumberOfDirectories
#define TIFFNumberOfStrips vtktiff_TIFFNumberOfStrips
#define TIFFNumberOfTiles vtktiff_TIFFNumberOfTiles
#define TIFFOpen vtktiff_TIFFOpen
#define TIFFOpenW vtktiff_TIFFOpenW
#define TIFFPredictorCleanup vtktiff_TIFFPredictorCleanup
#define TIFFPredictorInit vtktiff_TIFFPredictorInit
#define TIFFPrintDirectory vtktiff_TIFFPrintDirectory
#define TIFFRGBAImageBegin vtktiff_TIFFRGBAImageBegin
#define TIFFRGBAImageEnd vtktiff_TIFFRGBAImageEnd
#define TIFFRGBAImageGet vtktiff_TIFFRGBAImageGet
#define TIFFRGBAImageOK vtktiff_TIFFRGBAImageOK
#define TIFFRasterScanlineSize vtktiff_TIFFRasterScanlineSize
#define TIFFRasterScanlineSize64 vtktiff_TIFFRasterScanlineSize64
#define TIFFRawStripSize vtktiff_TIFFRawStripSize
#define TIFFRawStripSize64 vtktiff_TIFFRawStripSize64
#define TIFFReadBufferSetup vtktiff_TIFFReadBufferSetup
#define TIFFReadCustomDirectory vtktiff_TIFFReadCustomDirectory
#define TIFFReadDirectory vtktiff_TIFFReadDirectory
#define TIFFReadEXIFDirectory vtktiff_TIFFReadEXIFDirectory
#define TIFFReadEncodedStrip vtktiff_TIFFReadEncodedStrip
#define TIFFReadEncodedTile vtktiff_TIFFReadEncodedTile
#define TIFFReadFromUserBuffer vtktiff_TIFFReadFromUserBuffer
#define TIFFReadRGBAImage vtktiff_TIFFReadRGBAImage
#define TIFFReadRGBAImageOriented vtktiff_TIFFReadRGBAImageOriented
#define TIFFReadRGBAStrip vtktiff_TIFFReadRGBAStrip
#define TIFFReadRGBAStripExt vtktiff_TIFFReadRGBAStripExt
#define TIFFReadRGBATile vtktiff_TIFFReadRGBATile
#define TIFFReadRGBATileExt vtktiff_TIFFReadRGBATileExt
#define TIFFReadRawStrip vtktiff_TIFFReadRawStrip
#define TIFFReadRawTile vtktiff_TIFFReadRawTile
#define TIFFReadScanline vtktiff_TIFFReadScanline
#define TIFFReadTile vtktiff_TIFFReadTile
#define TIFFRegisterCODEC vtktiff_TIFFRegisterCODEC
#define TIFFReverseBits vtktiff_TIFFReverseBits
#define TIFFRewriteDirectory vtktiff_TIFFRewriteDirectory
#define TIFFScanlineSize vtktiff_TIFFScanlineSize
#define TIFFScanlineSize64 vtktiff_TIFFScanlineSize64
#define TIFFSetClientInfo vtktiff_TIFFSetClientInfo
#define TIFFSetClientdata vtktiff_TIFFSetClientdata
#define TIFFSetCompressionScheme vtktiff_TIFFSetCompressionScheme
#define TIFFSetDirectory vtktiff_TIFFSetDirectory
#define TIFFSetErrorHandler vtktiff_TIFFSetErrorHandler
#define TIFFSetErrorHandlerExt vtktiff_TIFFSetErrorHandlerExt
#define TIFFSetField vtktiff_TIFFSetField
#define TIFFSetFileName vtktiff_TIFFSetFileName
#define TIFFSetFileno vtktiff_TIFFSetFileno
#define TIFFSetMode vtktiff_TIFFSetMode
#define TIFFSetSubDirectory vtktiff_TIFFSetSubDirectory
#define TIFFSetTagExtender vtktiff_TIFFSetTagExtender
#define TIFFSetWarningHandler vtktiff_TIFFSetWarningHandler
#define TIFFSetWarningHandlerExt vtktiff_TIFFSetWarningHandlerExt
#define TIFFSetWriteOffset vtktiff_TIFFSetWriteOffset
#define TIFFSetupStrips vtktiff_TIFFSetupStrips
#define TIFFStripSize vtktiff_TIFFStripSize
#define TIFFStripSize64 vtktiff_TIFFStripSize64
#define TIFFSwabArrayOfDouble vtktiff_TIFFSwabArrayOfDouble
#define TIFFSwabArrayOfFloat vtktiff_TIFFSwabArrayOfFloat
#define TIFFSwabArrayOfLong vtktiff_TIFFSwabArrayOfLong
#define TIFFSwabArrayOfLong8 vtktiff_TIFFSwabArrayOfLong8
#define TIFFSwabArrayOfShort vtktiff_TIFFSwabArrayOfShort
#define TIFFSwabArrayOfTriples vtktiff_TIFFSwabArrayOfTriples
#define TIFFSwabDouble vtktiff_TIFFSwabDouble
#define TIFFSwabFloat vtktiff_TIFFSwabFloat
#define TIFFSwabLong vtktiff_TIFFSwabLong
#define TIFFSwabLong8 vtktiff_TIFFSwabLong8
#define TIFFSwabShort vtktiff_TIFFSwabShort
#define TIFFTileRowSize vtktiff_TIFFTileRowSize
#define TIFFTileRowSize64 vtktiff_TIFFTileRowSize64
#define TIFFTileSize vtktiff_TIFFTileSize
#define TIFFTileSize64 vtktiff_TIFFTileSize64
#define TIFFUnRegisterCODEC vtktiff_TIFFUnRegisterCODEC
#define TIFFUnlinkDirectory vtktiff_TIFFUnlinkDirectory
#define TIFFUnsetField vtktiff_TIFFUnsetField
#define TIFFVGetField vtktiff_TIFFVGetField
#define TIFFVGetFieldDefaulted vtktiff_TIFFVGetFieldDefaulted
#define TIFFVSetField vtktiff_TIFFVSetField
#define TIFFVStripSize vtktiff_TIFFVStripSize
#define TIFFVStripSize64 vtktiff_TIFFVStripSize64
#define TIFFVTileSize vtktiff_TIFFVTileSize
#define TIFFVTileSize64 vtktiff_TIFFVTileSize64
#define TIFFWarning vtktiff_TIFFWarning
#define TIFFWarningExt vtktiff_TIFFWarningExt
#define TIFFWriteBufferSetup vtktiff_TIFFWriteBufferSetup
#define TIFFWriteCheck vtktiff_TIFFWriteCheck
#define TIFFWriteCustomDirectory vtktiff_TIFFWriteCustomDirectory
#define TIFFWriteDirectory vtktiff_TIFFWriteDirectory
#define TIFFWriteEncodedStrip vtktiff_TIFFWriteEncodedStrip
#define TIFFWriteEncodedTile vtktiff_TIFFWriteEncodedTile
#define TIFFWriteRawStrip vtktiff_TIFFWriteRawStrip
#define TIFFWriteRawTile vtktiff_TIFFWriteRawTile
#define TIFFWriteScanline vtktiff_TIFFWriteScanline
#define TIFFWriteTile vtktiff_TIFFWriteTile
#define TIFFXYZToRGB vtktiff_TIFFXYZToRGB
#define TIFFYCbCrToRGBInit vtktiff_TIFFYCbCrToRGBInit
#define TIFFYCbCrtoRGB vtktiff_TIFFYCbCrtoRGB
#define XYZtoRGB24 vtktiff_XYZtoRGB24
#define _TIFFBuiltinCODECS vtktiff__TIFFBuiltinCODECS
#define _TIFFCastUInt64ToSSize vtktiff__TIFFCastUInt64ToSSize
#define _TIFFCheckFieldIsValidForCodec vtktiff__TIFFCheckFieldIsValidForCodec
#define _TIFFCheckMalloc vtktiff__TIFFCheckMalloc
#define _TIFFCheckRealloc vtktiff__TIFFCheckRealloc
#define _TIFFClampDoubleToFloat vtktiff__TIFFClampDoubleToFloat
#define _TIFFCreateAnonField vtktiff__TIFFCreateAnonField
#define _TIFFDataSize vtktiff__TIFFDataSize
#define _TIFFDefaultStripSize vtktiff__TIFFDefaultStripSize
#define _TIFFDefaultTileSize vtktiff__TIFFDefaultTileSize
#define _TIFFFax3fillruns vtktiff__TIFFFax3fillruns
#define _TIFFFillStriles vtktiff__TIFFFillStriles
#define _TIFFFindOrRegisterField vtktiff__TIFFFindOrRegisterField
#define _TIFFGetExifFields vtktiff__TIFFGetExifFields
#define _TIFFGetFields vtktiff__TIFFGetFields
#define _TIFFMergeFields vtktiff__TIFFMergeFields
#define _TIFFMultiply32 vtktiff__TIFFMultiply32
#define _TIFFMultiply64 vtktiff__TIFFMultiply64
#define _TIFFMultiplySSize vtktiff__TIFFMultiplySSize
#define _TIFFNoPostDecode vtktiff__TIFFNoPostDecode
#define _TIFFNoPreCode vtktiff__TIFFNoPreCode
#define _TIFFNoRowDecode vtktiff__TIFFNoRowDecode
#define _TIFFNoRowEncode vtktiff__TIFFNoRowEncode
#define _TIFFNoSeek vtktiff__TIFFNoSeek
#define _TIFFNoStripDecode vtktiff__TIFFNoStripDecode
#define _TIFFNoStripEncode vtktiff__TIFFNoStripEncode
#define _TIFFNoTileDecode vtktiff__TIFFNoTileDecode
#define _TIFFNoTileEncode vtktiff__TIFFNoTileEncode
#define _TIFFPrintFieldInfo vtktiff__TIFFPrintFieldInfo
#define _TIFFReadEncodedStripAndAllocBuffer vtktiff__TIFFReadEncodedStripAndAllocBuffer
#define _TIFFReadEncodedTileAndAllocBuffer vtktiff__TIFFReadEncodedTileAndAllocBuffer
#define _TIFFReadTileAndAllocBuffer vtktiff__TIFFReadTileAndAllocBuffer
#define _TIFFRewriteField vtktiff__TIFFRewriteField
#define _TIFFSeekOK vtktiff__TIFFSeekOK
#define _TIFFSetDefaultCompressionState vtktiff__TIFFSetDefaultCompressionState
#define _TIFFSetupFields vtktiff__TIFFSetupFields
#define _TIFFSwab16BitData vtktiff__TIFFSwab16BitData
#define _TIFFSwab24BitData vtktiff__TIFFSwab24BitData
#define _TIFFSwab32BitData vtktiff__TIFFSwab32BitData
#define _TIFFSwab64BitData vtktiff__TIFFSwab64BitData
#define _TIFFUInt64ToDouble vtktiff__TIFFUInt64ToDouble
#define _TIFFUInt64ToFloat vtktiff__TIFFUInt64ToFloat
#define _TIFFcalloc vtktiff__TIFFcalloc
#define _TIFFerrorHandler vtktiff__TIFFerrorHandler
#define _TIFFerrorHandlerExt vtktiff__TIFFerrorHandlerExt
#define _TIFFfree vtktiff__TIFFfree
#define _TIFFgetMode vtktiff__TIFFgetMode
#define _TIFFmalloc vtktiff__TIFFmalloc
#define _TIFFmemcmp vtktiff__TIFFmemcmp
#define _TIFFmemcpy vtktiff__TIFFmemcpy
#define _TIFFmemset vtktiff__TIFFmemset
#define _TIFFprintAscii vtktiff__TIFFprintAscii
#define _TIFFprintAsciiTag vtktiff__TIFFprintAsciiTag
#define _TIFFrealloc vtktiff__TIFFrealloc
#define _TIFFsetByteArray vtktiff__TIFFsetByteArray
#define _TIFFsetDoubleArray vtktiff__TIFFsetDoubleArray
#define _TIFFsetFloatArray vtktiff__TIFFsetFloatArray
#define _TIFFsetLongArray vtktiff__TIFFsetLongArray
#define _TIFFsetShortArray vtktiff__TIFFsetShortArray
#define _TIFFsetString vtktiff__TIFFsetString
#define _TIFFwarningHandler vtktiff__TIFFwarningHandler
#define _TIFFwarningHandlerExt vtktiff__TIFFwarningHandlerExt
#define libport_dummy_function vtktiff_libport_dummy_function
#define uv_decode vtktiff_uv_decode
#define uv_encode vtktiff_uv_encode

#endif
